package uk.ac.starlink.ttools.pipe;

import java.util.Iterator;
import uk.ac.starlink.table.StarTable;
import uk.ac.starlink.ttools.ArgException;

/**
 * Filter for picking only the last few rows of a table.
 *
 * @author   Mark Taylor (Starlink)
 * @since    8 Mar 2005
 */
public class TailFilter implements ProcessingFilter {

    public String getName() {
        return "tail";
    }

    public String getFilterUsage() {
        return "<nrows>";
    }

    public ProcessingStep createStep( Iterator argIt ) throws ArgException {
        if ( argIt.hasNext() ) {
            String countStr = (String) argIt.next();
            argIt.remove();
            long count;
            try {
                count = Long.parseLong( countStr );
            }
            catch ( NumberFormatException e ) {
                throw new ArgException( "Row count " + countStr + 
                                        " not numeric" );
            }
            if ( count < 0 ) {
                throw new ArgException( "Nrows must be >= 0" );
            }
            return new TailStep( count );
        }
        else {
            throw new ArgException( "No row count given" );
        }
    }

    private static class TailStep implements ProcessingStep {
        final long count_;

        TailStep( long count ) {
            count_ = count;
        }

        public StarTable wrap( StarTable base ) {
            return new TailTable( base, count_ );
        }
    }
}
